using APIMappers;
using Dto;
using HeartTrackAPI.Request;
using HeartTrackAPI.Responce;
using Microsoft.AspNetCore.Mvc;
using Shared;
using Model;
using Model.Manager;
using Model.Repository;

namespace HeartTrackAPI.Controllers;
[ApiController]
[Route("api/activities")]
public class ActivityController : Controller
{
    private readonly IActivityRepository _activityService;
    private readonly ILogger<ActivityController> _logger;
    private readonly IActivityManager _activityManager;

    public ActivityController(IDataManager dataManager, ILogger<ActivityController> logger)
    {
        _activityService = dataManager.ActivityRepo;
        _logger = logger;
    }

    [HttpGet]
    [ProducesResponseType(typeof(PageResponse<ActivityDto>), 200)]
    [ProducesResponseType(400)]
    [ProducesResponseType(500)]
    public async Task<ActionResult<PageResponse<ActivityDto>>> GetActivities([FromQuery] PageRequest pageRequest)
    {
        try
        {
            var totalCount = await _activityService.GetNbItems();
            if (pageRequest.Count * pageRequest.Index >= totalCount)
            {
                _logger.LogError("To many object is asked the max is {totalCount} but the request is superior of ", totalCount);
                return BadRequest("To many object is asked the max is : " + totalCount);
            }
            _logger.LogInformation("Executing {Action} with parameters: {Parameters}", nameof(GetActivities), pageRequest);
            var activities = await _activityService.GetActivities(pageRequest.Index, pageRequest.Count, ActivityOrderCriteria.None, pageRequest.Descending ?? false);
            var pageResponse = new PageResponse<ActivityDto>(pageRequest.Index, pageRequest.Count, totalCount, activities.Select(a => a.ToDto()));
            return Ok(pageResponse);
            return Ok();
        }
        catch (Exception e)
        {
            _logger.LogError(e, "Error while getting all activities");
            return StatusCode(500);
        }
    }
    /*
    public async Task<IActionResult> PostFitFile([FromForm] IFormFile file)
    {
        if (file == null)
        {
            return BadRequest("No file was provided");
        }
        var activity = await _activityManager.AddActivityFromFitFile(file);
        à l'intérieur du   AddActivityFromFitFile ya un truc comme     var result = await _activityRepo.AddActivity(activity);

        if (activity == null)
        {
            return BadRequest("The file provided is not a valid fit file");
        }
        return CreatedAtAction(nameof(GetActivity), new { id = result.Id }, result.ToDto());

        return CreatedAtAction(nameof(GetActivities), activity.ToDto());
    }*/
    
    [HttpGet("{id}")]
    public async Task<ActionResult<ActivityDto>> GetActivity(int id)
    {
        var activity = await _activityService.GetActivityByIdAsync(id);
        if (activity == null)
        {
            return NotFound();
        }
        return Ok(activity.ToDto());
    }

    [HttpPut("{id}")]
    public async Task<IActionResult> PutActivity(int id, ActivityDto activityDto)
    {
        if (id != activityDto.Id)
        {
            return BadRequest();
        }
        var activity = activityDto.ToModel();
        var result = await _activityService.UpdateActivity(id, activity);
        if (result == null)
        {
            return NotFound();
        }
        return NoContent();
    }

    [HttpDelete("{id}")]
    public async Task<IActionResult> DeleteActivity(int id)
    {
        var result = await _activityService.DeleteActivity(id);
        if (!result)
        {
            return NotFound();
        }
        return NoContent();
    }
}